@@ -87,6 +87,15 @@ def _make_request(token, method_name, method='get', params=None, files=None):
     logger.debug("Request: method={0} url={1} params={2} files={3}".format(method, request_url, params, files).replace(token, token.split(':')[0] + ":{TOKEN}"))
     read_timeout = READ_TIMEOUT
     connect_timeout = CONNECT_TIMEOUT
+
+    if files:
+        files_copy = dict(files)
+        # process types.InputFile
+        for key, value in files_copy.items():
+            if isinstance(value, types.InputFile):
+                files[key] = value.file
+                
+    
     if files and format_header_param:
         fields.format_header_param = _no_encode(format_header_param)
     if params:
@@ -268,7 +277,7 @@ def send_message(
 
 
 def set_webhook(token, url=None, certificate=None, max_connections=None, allowed_updates=None, ip_address=None,
-                drop_pending_updates = None, timeout=None):
+                drop_pending_updates = None, timeout=None, secret_token=None):
     method_url = r'setWebhook'
     payload = {
         'url': url if url else "",
@@ -286,6 +295,8 @@ def set_webhook(token, url=None, certificate=None, max_connections=None, allowed
         payload['drop_pending_updates'] = drop_pending_updates
     if timeout:
         payload['timeout'] = timeout
+    if secret_token:
+        payload['secret_token'] = secret_token
     return _make_request(token, method_url, params=payload, files=files)
 
 
@@ -1550,7 +1561,10 @@ def get_sticker_set(token, name):
     method_url = 'getStickerSet'
     return _make_request(token, method_url, params={'name': name})
 
-
+def get_custom_emoji_stickers(token, custom_emoji_ids):
+    method_url = r'getCustomEmojiStickers'
+    return _make_request(token, method_url, params={'custom_emoji_ids': custom_emoji_ids})
+    
 def upload_sticker_file(token, user_id, png_sticker):
     method_url = 'uploadStickerFile'
     payload = {'user_id': user_id}
@@ -1560,7 +1574,7 @@ def upload_sticker_file(token, user_id, png_sticker):
 
 def create_new_sticker_set(
         token, user_id, name, title, emojis, png_sticker, tgs_sticker,
-        contains_masks=None, mask_position=None, webm_sticker=None):
+        mask_position=None, webm_sticker=None, sticker_type=None):
     method_url = 'createNewStickerSet'
     payload = {'user_id': user_id, 'name': name, 'title': title, 'emojis': emojis}
     if png_sticker:
@@ -1575,12 +1589,12 @@ def create_new_sticker_set(
         files = {stype: sticker}
     else:
         payload[stype] = sticker
-    if contains_masks is not None:
-        payload['contains_masks'] = contains_masks
     if mask_position:
         payload['mask_position'] = mask_position.to_json()
     if webm_sticker:
         payload['webm_sticker'] = webm_sticker
+    if sticker_type:
+        payload['sticker_type'] = sticker_type
     return _make_request(token, method_url, params=payload, files=files, method='post')
 
 
@@ -1622,6 +1636,45 @@ def answer_web_app_query(token, web_app_query_id, result: types.InlineQueryResul
     return _make_request(token, method_url, params=payload, method='post')
 
 
+def create_invoice_link(token, title, description, payload, provider_token,
+            currency, prices, max_tip_amount=None, suggested_tip_amounts=None, provider_data=None,
+            photo_url=None, photo_size=None, photo_width=None, photo_height=None, need_name=None, need_phone_number=None,
+            need_email=None, need_shipping_address=None, send_phone_number_to_provider=None,
+            send_email_to_provider=None, is_flexible=None):
+    method_url = r'createInvoiceLink'
+    payload = {'title': title, 'description': description, 'payload': payload, 'provider_token': provider_token,
+                'currency': currency, 'prices': _convert_list_json_serializable(prices)}
+    if max_tip_amount:
+        payload['max_tip_amount'] = max_tip_amount
+    if suggested_tip_amounts:
+        payload['suggested_tip_amounts'] = json.dumps(suggested_tip_amounts)
+    if provider_data:
+        payload['provider_data'] = provider_data
+    if photo_url:
+        payload['photo_url'] = photo_url
+    if photo_size:
+        payload['photo_size'] = photo_size
+    if photo_width:
+        payload['photo_width'] = photo_width
+    if photo_height:
+        payload['photo_height'] = photo_height
+    if need_name is not None:
+        payload['need_name'] = need_name
+    if need_phone_number is not None:
+        payload['need_phone_number'] = need_phone_number
+    if need_email is not None:
+        payload['need_email'] = need_email
+    if need_shipping_address is not None:
+        payload['need_shipping_address'] = need_shipping_address
+    if send_phone_number_to_provider is not None:
+        payload['send_phone_number_to_provider'] = send_phone_number_to_provider
+    if send_email_to_provider is not None:
+        payload['send_email_to_provider'] = send_email_to_provider
+    if is_flexible is not None:
+        payload['is_flexible'] = is_flexible
+    return _make_request(token, method_url, params=payload, method='post')
+
+
 # noinspection PyShadowingBuiltins
 def send_poll(
         token, chat_id,
