@@ -1,5 +1,7 @@
 import asyncio # for future uses
+import ssl
 import aiohttp
+import certifi
 from telebot import types
 
 try:
@@ -19,24 +21,29 @@ session = None
 
 FILE_URL = None
 
-CONNECT_TIMEOUT = 15
-READ_TIMEOUT = 30
-
-LONG_POLLING_TIMEOUT = 10 # Should be positive, short polling should be used for testing purposes only (https://core.telegram.org/bots/api#getupdates)
-REQUEST_TIMEOUT = 10
+REQUEST_TIMEOUT = None
 MAX_RETRIES = 3
 
 REQUEST_LIMIT = 50
 
 class SessionManager:
     def __init__(self) -> None:
-        self.session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(limit=REQUEST_LIMIT))
+        self.session = None
+        self.ssl_context = ssl.create_default_context(cafile=certifi.where())
+
 
     async def create_session(self):
-        self.session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(limit=REQUEST_LIMIT))
+        self.session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(
+            limit=REQUEST_LIMIT,
+            ssl_context=self.ssl_context
+        ))
         return self.session
 
     async def get_session(self):
+        if self.session is None:
+            self.session = await self.create_session()
+            return self.session
+            
         if self.session.closed:
             self.session = await self.create_session()
 
@@ -50,7 +57,7 @@ class SessionManager:
 session_manager = SessionManager()
 
 async def _process_request(token, url, method='get', params=None, files=None, request_timeout=None):
-    params = prepare_data(params, files)
+    params = _prepare_data(params, files)
     if request_timeout is None:
         request_timeout = REQUEST_TIMEOUT
     timeout = aiohttp.ClientTimeout(total=request_timeout)
@@ -61,10 +68,11 @@ async def _process_request(token, url, method='get', params=None, files=None, re
         current_try +=1
         try:
             async with session.request(method=method, url=API_URL.format(token, url), data=params, timeout=timeout, proxy=proxy) as resp:
+                got_result = True
                 logger.debug("Request: method={0} url={1} params={2} files={3} request_timeout={4} current_try={5}".format(method, url, params, files, request_timeout, current_try).replace(token, token.split(':')[0] + ":{TOKEN}"))
+                
                 json_result = await _check_result(url, resp)
                 if json_result:
-                    got_result = True
                     return json_result['result']
         except (ApiTelegramException,ApiInvalidJSONException, ApiHTTPException) as e:
             raise e
@@ -75,24 +83,17 @@ async def _process_request(token, url, method='get', params=None, files=None, re
         if not got_result:
             raise RequestTimeout("Request timeout. Request: method={0} url={1} params={2} files={3} request_timeout={4}".format(method, url, params, files, request_timeout, current_try))
         
-
-
-
-def prepare_file(obj):
+def _prepare_file(obj):
     """
-    returns os.path.basename for a given file
-
-    :param obj:
-    :return:
+    Prepares file for upload.
     """
     name = getattr(obj, 'name', None)
     if name and isinstance(name, str) and name[0] != '<' and name[-1] != '>':
         return os.path.basename(name)
 
-
-def prepare_data(params=None, files=None):
+def _prepare_data(params=None, files=None):
     """
-    prepare data for request.
+    Adds the parameters and files to the request.
 
     :param params:
     :param files:
@@ -103,18 +104,20 @@ def prepare_data(params=None, files=None):
     if params:
         for key, value in params.items():
             data.add_field(key, str(value))
-
     if files:
         for key, f in files.items():
             if isinstance(f, tuple):
                 if len(f) == 2:
-                    filename, fileobj = f
+                    file_name, file = f
                 else:
                     raise ValueError('Tuple must have exactly 2 elements: filename, fileobj')
+            elif isinstance(f, types.InputFile):
+                file_name = f.file_name
+                file = f.file
             else:
-                filename, fileobj = prepare_file(f) or key, f
+                file_name, file = _prepare_file(f) or key, f
 
-            data.add_field(key, fileobj, filename=filename)
+            data.add_field(key, file, filename=file_name)
 
     return data
 
@@ -147,10 +150,10 @@ async def get_file(token, file_id):
 
 async def get_file_url(token, file_id):
     if FILE_URL is None:
-        return "https://api.telegram.org/file/bot{0}/{1}".format(token, get_file(token, file_id)['file_path'])
+        return "https://api.telegram.org/file/bot{0}/{1}".format(token, await get_file(token, file_id)['file_path'])
     else:
         # noinspection PyUnresolvedReferences
-        return FILE_URL.format(token, get_file(token, file_id)['file_path'])
+        return FILE_URL.format(token, await get_file(token, file_id)['file_path'])
 
 
 async def download_file(token, file_path):
@@ -169,7 +172,7 @@ async def download_file(token, file_path):
 
 
 async def set_webhook(token, url=None, certificate=None, max_connections=None, allowed_updates=None, ip_address=None,
-                drop_pending_updates = None, timeout=None):
+                drop_pending_updates = None, timeout=None, secret_token=None):
     method_url = r'setWebhook'
     payload = {
         'url': url if url else "",
@@ -187,6 +190,8 @@ async def set_webhook(token, url=None, certificate=None, max_connections=None, a
         payload['drop_pending_updates'] = drop_pending_updates
     if timeout:
         payload['timeout'] = timeout
+    if secret_token:
+        payload['secret_token'] = secret_token
     return await _process_request(token, method_url, params=payload, files=files)
 
 
@@ -215,11 +220,11 @@ async def get_updates(token, offset=None, limit=None,
     params = {}
     if offset:
         params['offset'] = offset
-    elif limit:
+    if limit:
         params['limit'] = limit
-    elif timeout:
+    if timeout:
         params['timeout'] = timeout
-    elif allowed_updates:
+    if allowed_updates:
         params['allowed_updates'] = allowed_updates
     return await _process_request(token, method_name, params=params, request_timeout=request_timeout)
 
@@ -1527,6 +1532,9 @@ async def get_sticker_set(token, name):
     method_url = 'getStickerSet'
     return await _process_request(token, method_url, params={'name': name})
 
+async def get_custom_emoji_stickers(token, custom_emoji_ids):
+    method_url = r'getCustomEmojiStickers'
+    return await _process_request(token, method_url, params={'custom_emoji_ids': custom_emoji_ids})
 
 async def upload_sticker_file(token, user_id, png_sticker):
     method_url = 'uploadStickerFile'
@@ -1537,7 +1545,7 @@ async def upload_sticker_file(token, user_id, png_sticker):
 
 async def create_new_sticker_set(
         token, user_id, name, title, emojis, png_sticker, tgs_sticker,
-        contains_masks=None, mask_position=None, webm_sticker=None):
+        mask_position=None, webm_sticker=None, sticker_type=None):
     method_url = 'createNewStickerSet'
     payload = {'user_id': user_id, 'name': name, 'title': title, 'emojis': emojis}
     if png_sticker:
@@ -1552,12 +1560,12 @@ async def create_new_sticker_set(
         files = {stype: sticker}
     else:
         payload[stype] = sticker
-    if contains_masks is not None:
-        payload['contains_masks'] = contains_masks
     if mask_position:
         payload['mask_position'] = mask_position.to_json()
     if webm_sticker:
         payload['webm_sticker'] = webm_sticker
+    if sticker_type:
+        payload['sticker_type'] = sticker_type
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
@@ -1597,6 +1605,47 @@ async def delete_sticker_from_set(token, sticker):
     return await _process_request(token, method_url, params=payload, method='post')
 
 
+
+async def create_invoice_link(token, title, description, payload, provider_token,
+            currency, prices, max_tip_amount=None, suggested_tip_amounts=None, provider_data=None,
+            photo_url=None, photo_size=None, photo_width=None, photo_height=None, need_name=None, need_phone_number=None,
+            need_email=None, need_shipping_address=None, send_phone_number_to_provider=None,
+            send_email_to_provider=None, is_flexible=None):
+    method_url = r'createInvoiceLink'
+    payload = {'title': title, 'description': description, 'payload': payload, 'provider_token': provider_token,
+                'currency': currency, 'prices': await _convert_list_json_serializable(prices)}
+    if max_tip_amount:
+        payload['max_tip_amount'] = max_tip_amount
+    if suggested_tip_amounts:
+        payload['suggested_tip_amounts'] = json.dumps(suggested_tip_amounts)
+    if provider_data:
+        payload['provider_data'] = provider_data
+    if photo_url:
+        payload['photo_url'] = photo_url
+    if photo_size:
+        payload['photo_size'] = photo_size
+    if photo_width:
+        payload['photo_width'] = photo_width
+    if photo_height:
+        payload['photo_height'] = photo_height
+    if need_name is not None:
+        payload['need_name'] = need_name
+    if need_phone_number is not None:
+        payload['need_phone_number'] = need_phone_number
+    if need_email is not None:
+        payload['need_email'] = need_email
+    if need_shipping_address is not None:
+        payload['need_shipping_address'] = need_shipping_address
+    if send_phone_number_to_provider is not None:
+        payload['send_phone_number_to_provider'] = send_phone_number_to_provider
+    if send_email_to_provider is not None:
+        payload['send_email_to_provider'] = send_email_to_provider
+    if is_flexible is not None:
+        payload['is_flexible'] = is_flexible
+    return await _process_request(token, method_url, params=payload, method='post')
+
+
+
 # noinspection PyShadowingBuiltins
 async def send_poll(
         token, chat_id,
@@ -1771,9 +1820,10 @@ class ApiTelegramException(ApiException):
             result)
         self.result_json = result_json
         self.error_code = result_json['error_code']
+        self.description = result_json['description']
 
 class RequestTimeout(Exception):
     """
     This class represents a request timeout.
     """
-    pass
\ No newline at end of file
+    pass
